import struct

from .error import *
from .zehnder_pb2 import *


class Message(object):
    class_to_type = {
        SetAddressRequest: GatewayOperation.SetAddressRequestType,
        RegisterAppRequest: GatewayOperation.RegisterAppRequestType,
        StartSessionRequest: GatewayOperation.StartSessionRequestType,
        CloseSessionRequest: GatewayOperation.CloseSessionRequestType,
        ListRegisteredAppsRequest: GatewayOperation.ListRegisteredAppsRequestType,
        DeregisterAppRequest: GatewayOperation.DeregisterAppRequestType,
        ChangePinRequest: GatewayOperation.ChangePinRequestType,
        GetRemoteAccessIdRequest: GatewayOperation.GetRemoteAccessIdRequestType,
        SetRemoteAccessIdRequest: GatewayOperation.SetRemoteAccessIdRequestType,
        GetSupportIdRequest: GatewayOperation.GetSupportIdRequestType,
        SetSupportIdRequest: GatewayOperation.SetSupportIdRequestType,
        GetWebIdRequest: GatewayOperation.GetWebIdRequestType,
        SetWebIdRequest: GatewayOperation.SetWebIdRequestType,
        SetPushIdRequest: GatewayOperation.SetPushIdRequestType,
        DebugRequest: GatewayOperation.DebugRequestType,
        UpgradeRequest: GatewayOperation.UpgradeRequestType,
        SetDeviceSettingsRequest: GatewayOperation.SetDeviceSettingsRequestType,
        VersionRequest: GatewayOperation.VersionRequestType,
        SetAddressConfirm: GatewayOperation.SetAddressConfirmType,
        RegisterAppConfirm: GatewayOperation.RegisterAppConfirmType,
        StartSessionConfirm: GatewayOperation.StartSessionConfirmType,
        CloseSessionConfirm: GatewayOperation.CloseSessionConfirmType,
        ListRegisteredAppsConfirm: GatewayOperation.ListRegisteredAppsConfirmType,
        DeregisterAppConfirm: GatewayOperation.DeregisterAppConfirmType,
        ChangePinConfirm: GatewayOperation.ChangePinConfirmType,
        GetRemoteAccessIdConfirm: GatewayOperation.GetRemoteAccessIdConfirmType,
        SetRemoteAccessIdConfirm: GatewayOperation.SetRemoteAccessIdConfirmType,
        GetSupportIdConfirm: GatewayOperation.GetSupportIdConfirmType,
        SetSupportIdConfirm: GatewayOperation.SetSupportIdConfirmType,
        GetWebIdConfirm: GatewayOperation.GetWebIdConfirmType,
        SetWebIdConfirm: GatewayOperation.SetWebIdConfirmType,
        SetPushIdConfirm: GatewayOperation.SetPushIdConfirmType,
        DebugConfirm: GatewayOperation.DebugConfirmType,
        UpgradeConfirm: GatewayOperation.UpgradeConfirmType,
        SetDeviceSettingsConfirm: GatewayOperation.SetDeviceSettingsConfirmType,
        VersionConfirm: GatewayOperation.VersionConfirmType,
        GatewayNotification: GatewayOperation.GatewayNotificationType,
        KeepAlive: GatewayOperation.KeepAliveType,
        FactoryReset: GatewayOperation.FactoryResetType,
        CnTimeRequest: GatewayOperation.CnTimeRequestType,
        CnTimeConfirm: GatewayOperation.CnTimeConfirmType,
        CnNodeRequest: GatewayOperation.CnNodeRequestType,
        CnNodeNotification: GatewayOperation.CnNodeNotificationType,
        CnRmiRequest: GatewayOperation.CnRmiRequestType,
        CnRmiResponse: GatewayOperation.CnRmiResponseType,
        CnRmiAsyncRequest: GatewayOperation.CnRmiAsyncRequestType,
        CnRmiAsyncConfirm: GatewayOperation.CnRmiAsyncConfirmType,
        CnRmiAsyncResponse: GatewayOperation.CnRmiAsyncResponseType,
        CnRpdoRequest: GatewayOperation.CnRpdoRequestType,
        CnRpdoConfirm: GatewayOperation.CnRpdoConfirmType,
        CnRpdoNotification: GatewayOperation.CnRpdoNotificationType,
        CnAlarmNotification: GatewayOperation.CnAlarmNotificationType,
        CnFupReadRegisterRequest: GatewayOperation.CnFupReadRegisterRequestType,
        CnFupReadRegisterConfirm: GatewayOperation.CnFupReadRegisterConfirmType,
        CnFupProgramBeginRequest: GatewayOperation.CnFupProgramBeginRequestType,
        CnFupProgramBeginConfirm: GatewayOperation.CnFupProgramBeginConfirmType,
        CnFupProgramRequest: GatewayOperation.CnFupProgramRequestType,
        CnFupProgramConfirm: GatewayOperation.CnFupProgramConfirmType,
        CnFupProgramEndRequest: GatewayOperation.CnFupProgramEndRequestType,
        CnFupProgramEndConfirm: GatewayOperation.CnFupProgramEndConfirmType,
        CnFupReadRequest: GatewayOperation.CnFupReadRequestType,
        CnFupReadConfirm: GatewayOperation.CnFupReadConfirmType,
        CnFupResetRequest: GatewayOperation.CnFupResetRequestType,
        CnFupResetConfirm: GatewayOperation.CnFupResetConfirmType,
    }

    class_to_confirm = {
        SetAddressRequest: SetAddressConfirm,
        RegisterAppRequest: RegisterAppConfirm,
        StartSessionRequest: StartSessionConfirm,
        CloseSessionRequest: CloseSessionConfirm,
        ListRegisteredAppsRequest: ListRegisteredAppsConfirm,
        DeregisterAppRequest: DeregisterAppConfirm,
        ChangePinRequest: ChangePinConfirm,
        GetRemoteAccessIdRequest: GetRemoteAccessIdConfirm,
        SetRemoteAccessIdRequest: SetRemoteAccessIdConfirm,
        GetSupportIdRequest: GetSupportIdConfirm,
        SetSupportIdRequest: SetSupportIdConfirm,
        GetWebIdRequest: GetWebIdConfirm,
        SetWebIdRequest: SetWebIdConfirm,
        SetPushIdRequest: SetPushIdConfirm,
        DebugRequest: DebugConfirm,
        UpgradeRequest: UpgradeConfirm,
        SetDeviceSettingsRequest: SetDeviceSettingsConfirm,
        VersionRequest: VersionConfirm,
        CnTimeRequest: CnTimeConfirm,
        CnRmiRequest: CnRmiResponse,
        CnRmiAsyncRequest: CnRmiAsyncConfirm,
        CnRpdoRequest: CnRpdoConfirm,
        CnFupReadRegisterRequest: CnFupReadRegisterConfirm,
        CnFupProgramBeginRequest: CnFupProgramBeginConfirm,
        CnFupProgramRequest: CnFupProgramConfirm,
        CnFupProgramEndRequest: CnFupProgramEndConfirm,
        CnFupReadRequest: CnFupReadConfirm,
        CnFupResetRequest: CnFupResetConfirm,
    }

    request_type_to_class_mapping = {
        GatewayOperation.SetAddressRequestType: SetAddressRequest,
        GatewayOperation.RegisterAppRequestType: RegisterAppRequest,
        GatewayOperation.StartSessionRequestType: StartSessionRequest,
        GatewayOperation.CloseSessionRequestType: CloseSessionRequest,
        GatewayOperation.ListRegisteredAppsRequestType: ListRegisteredAppsRequest,
        GatewayOperation.DeregisterAppRequestType: DeregisterAppRequest,
        GatewayOperation.ChangePinRequestType: ChangePinRequest,
        GatewayOperation.GetRemoteAccessIdRequestType: GetRemoteAccessIdRequest,
        GatewayOperation.SetRemoteAccessIdRequestType: SetRemoteAccessIdRequest,
        GatewayOperation.GetSupportIdRequestType: GetSupportIdRequest,
        GatewayOperation.SetSupportIdRequestType: SetSupportIdRequest,
        GatewayOperation.GetWebIdRequestType: GetWebIdRequest,
        GatewayOperation.SetWebIdRequestType: SetWebIdRequest,
        GatewayOperation.SetPushIdRequestType: SetPushIdRequest,
        GatewayOperation.DebugRequestType: DebugRequest,
        GatewayOperation.UpgradeRequestType: UpgradeRequest,
        GatewayOperation.SetDeviceSettingsRequestType: SetDeviceSettingsRequest,
        GatewayOperation.VersionRequestType: VersionRequest,
        GatewayOperation.SetAddressConfirmType: SetAddressConfirm,
        GatewayOperation.RegisterAppConfirmType: RegisterAppConfirm,
        GatewayOperation.StartSessionConfirmType: StartSessionConfirm,
        GatewayOperation.CloseSessionConfirmType: CloseSessionConfirm,
        GatewayOperation.ListRegisteredAppsConfirmType: ListRegisteredAppsConfirm,
        GatewayOperation.DeregisterAppConfirmType: DeregisterAppConfirm,
        GatewayOperation.ChangePinConfirmType: ChangePinConfirm,
        GatewayOperation.GetRemoteAccessIdConfirmType: GetRemoteAccessIdConfirm,
        GatewayOperation.SetRemoteAccessIdConfirmType: SetRemoteAccessIdConfirm,
        GatewayOperation.GetSupportIdConfirmType: GetSupportIdConfirm,
        GatewayOperation.SetSupportIdConfirmType: SetSupportIdConfirm,
        GatewayOperation.GetWebIdConfirmType: GetWebIdConfirm,
        GatewayOperation.SetWebIdConfirmType: SetWebIdConfirm,
        GatewayOperation.SetPushIdConfirmType: SetPushIdConfirm,
        GatewayOperation.DebugConfirmType: DebugConfirm,
        GatewayOperation.UpgradeConfirmType: UpgradeConfirm,
        GatewayOperation.SetDeviceSettingsConfirmType: SetDeviceSettingsConfirm,
        GatewayOperation.VersionConfirmType: VersionConfirm,
        GatewayOperation.GatewayNotificationType: GatewayNotification,
        GatewayOperation.KeepAliveType: KeepAlive,
        GatewayOperation.FactoryResetType: FactoryReset,
        GatewayOperation.CnTimeRequestType: CnTimeRequest,
        GatewayOperation.CnTimeConfirmType: CnTimeConfirm,
        GatewayOperation.CnNodeRequestType: CnNodeRequest,
        GatewayOperation.CnNodeNotificationType: CnNodeNotification,
        GatewayOperation.CnRmiRequestType: CnRmiRequest,
        GatewayOperation.CnRmiResponseType: CnRmiResponse,
        GatewayOperation.CnRmiAsyncRequestType: CnRmiAsyncRequest,
        GatewayOperation.CnRmiAsyncConfirmType: CnRmiAsyncConfirm,
        GatewayOperation.CnRmiAsyncResponseType: CnRmiAsyncResponse,
        GatewayOperation.CnRpdoRequestType: CnRpdoRequest,
        GatewayOperation.CnRpdoConfirmType: CnRpdoConfirm,
        GatewayOperation.CnRpdoNotificationType: CnRpdoNotification,
        GatewayOperation.CnAlarmNotificationType: CnAlarmNotification,
        GatewayOperation.CnFupReadRegisterRequestType: CnFupReadRegisterRequest,
        GatewayOperation.CnFupReadRegisterConfirmType: CnFupReadRegisterConfirm,
        GatewayOperation.CnFupProgramBeginRequestType: CnFupProgramBeginRequest,
        GatewayOperation.CnFupProgramBeginConfirmType: CnFupProgramBeginConfirm,
        GatewayOperation.CnFupProgramRequestType: CnFupProgramRequest,
        GatewayOperation.CnFupProgramConfirmType: CnFupProgramConfirm,
        GatewayOperation.CnFupProgramEndRequestType: CnFupProgramEndRequest,
        GatewayOperation.CnFupProgramEndConfirmType: CnFupProgramEndConfirm,
        GatewayOperation.CnFupReadRequestType: CnFupReadRequest,
        GatewayOperation.CnFupReadConfirmType: CnFupReadConfirm,
        GatewayOperation.CnFupResetRequestType: CnFupResetRequest,
        GatewayOperation.CnFupResetConfirmType: CnFupResetConfirm,
    }

    def __init__(self, cmd, msg, src, dst):
        self.cmd = cmd
        self.msg = msg
        self.src = src
        self.dst = dst

    @classmethod
    def create(cls, src, dst, command, cmd_params=None, msg_params=None):

        cmd = GatewayOperation()
        cmd.type = cls.class_to_type[command]
        if cmd_params is not None:
            for param in cmd_params:
                if cmd_params[param] is not None:
                    setattr(cmd, param, cmd_params[param])

        msg = command()
        if msg_params is not None:
            for param in msg_params:
                if msg_params[param] is not None:
                    setattr(msg, param, msg_params[param])

        return Message(cmd, msg, src, dst)

    def __str__(self):
        return "%s -> %s: %s %s\n%s\n%s" % (
            self.src.hex(),
            self.dst.hex(),
            self.cmd.SerializeToString().hex(),
            self.msg.SerializeToString().hex(),
            self.cmd,
            self.msg,
        )

    def encode(self):
        cmd_buf = self.cmd.SerializeToString()
        msg_buf = self.msg.SerializeToString()
        cmd_len_buf = struct.pack('>H', len(cmd_buf))
        msg_len_buf = struct.pack('>L', 16 + 16 + 2 + len(cmd_buf) + len(msg_buf))

        return msg_len_buf + self.src + self.dst + cmd_len_buf + cmd_buf + msg_buf

    @classmethod
    def decode(cls, packet):

        src_buf = packet[4:20]
        dst_buf = packet[20:36]
        cmd_len = struct.unpack('>H', packet[36:38])[0]
        cmd_buf = packet[38:38 + cmd_len]
        msg_buf = packet[38 + cmd_len:]

        # Parse command
        cmd = GatewayOperation()
        cmd.ParseFromString(cmd_buf)

        # Parse message
        cmd_type = cls.request_type_to_class_mapping.get(cmd.type)
        msg = cmd_type()
        msg.ParseFromString(msg_buf)

        return Message(cmd, msg, src_buf, dst_buf)



#!/usr/bin/python3
import struct
import sys

import pycomfoconnect
from pycomfoconnect import zehnder_pb2

"""
Pipe the messages to this programm to decode them. If they are stored in HEX, you should pass them to xxd -r -p first.
You can combine both input and output streams.

Usage:
  $ cat captures/raw_*.txt | xxd -r -p | ./manually_decode.py
"""

pdids = {}
rmis = {}

while True:

    # Read packet from file
    msg_len_buf = sys.stdin.buffer.read(4)
    if not msg_len_buf:
        break
    msg_len = struct.unpack('>L', msg_len_buf)[0]
    msg_buf = sys.stdin.buffer.read(msg_len)

    # Decode packet
    message = pycomfoconnect.Message.decode(msg_len_buf + msg_buf)

    if message.cmd.type == zehnder_pb2.GatewayOperation.CnRmiRequestType:
        if not message.cmd.reference in rmis:
            rmis[message.cmd.reference] = {}
        rmis[message.cmd.reference]['tx'] = message.msg.message.hex()

    if message.cmd.type == zehnder_pb2.GatewayOperation.CnRmiResponseType:
        if not message.cmd.reference in rmis:
            rmis[message.cmd.reference] = {}
        rmis[message.cmd.reference]['rx'] = message.msg.message.hex()

    if message.cmd.type == zehnder_pb2.GatewayOperation.CnRpdoRequestType:
        if not message.msg.pdid in pdids:
            pdids[message.msg.pdid] = {}
        try:
            pdids[message.msg.pdid]['tx'].append(message.msg.type)
        except KeyError:
            pdids[message.msg.pdid]['tx'] = [message.msg.type]

    if message.cmd.type == zehnder_pb2.GatewayOperation.CnRpdoConfirmType:
        pass

    if message.cmd.type == zehnder_pb2.GatewayOperation.CnRpdoNotificationType:
        if not message.msg.pdid in pdids:
            pdids[message.msg.pdid] = {}
        try:
            pdids[message.msg.pdid]['rx'].append(message.msg.data.hex())
        except KeyError:
            pdids[message.msg.pdid]['rx'] = [message.msg.data.hex()]

print("CnRpdoRequestType")
for pdid in pdids:
    print(pdid, pdids[pdid])
#
# # print("CnRmiRequestType")
# # for rmi in rmis:
# #     print(rmi, rmis[rmi])
#



# Commands
CMD_FAN_MODE_AWAY = b'\x84\x15\x01\x01\x00\x00\x00\x00\x01\x00\x00\x00\x00'
CMD_FAN_MODE_LOW = b'\x84\x15\x01\x01\x00\x00\x00\x00\x01\x00\x00\x00\x01'
CMD_FAN_MODE_MEDIUM = b'\x84\x15\x01\x01\x00\x00\x00\x00\x01\x00\x00\x00\x02'
CMD_FAN_MODE_HIGH = b'\x84\x15\x01\x01\x00\x00\x00\x00\x01\x00\x00\x00\x03'
CMD_MODE_AUTO = b'\x85\x15\x08\x01'
CMD_MODE_MANUAL = b'\x84\x15\x08\x01\x00\x00\x00\x00\x01\x00\x00\x00\x01'
CMD_VENTMODE_SUPPLY = b'\x84\x15\x06\x01\x00\x00\x00\x00\x10\x0e\x00\x00\x01'
CMD_VENTMODE_BALANCE = b'\x85\x15\x06\x01'
CMD_VENTMODE_STOP_SUPPLY_FAN_1  = b'\x84\x15\x07\x01\x00\x00\x00\x00\x10\x0e\x00\x00\x01' # Stoppt Supply-Fan für 1h
CMD_START_SUPPLY_FAN            = b'\x85\x15\x07\x01'
CMD_VENTMODE_STOP_EXHAUST_FAN_1 = b'\x84\x15\x06\x01\x00\x00\x00\x00\x10\x0e\x00\x00\x01' # Stoppt Exhaust-Fan für 1h
CMD_START_EXHAUST_FAN           = b'\x85\x15\x06\x01'
CMD_TEMPPROF_NORMAL = b'\x84\x15\x03\x01\x00\x00\x00\x00\xff\xff\xff\xff\x00'
CMD_TEMPPROF_COOL = b'\x84\x15\x03\x01\x00\x00\x00\x00\xff\xff\xff\xff\x01'
CMD_TEMPPROF_WARM = b'\x84\x15\x03\x01\x00\x00\x00\x00\xff\xff\xff\xff\x02'
CMD_BYPASS_ON = b'\x84\x15\x02\x01\x00\x00\x00\x00\x10\x0e\x00\x00\x01'
CMD_BYPASS_OFF = b'\x84\x15\x02\x01\x00\x00\x00\x00\x10\x0e\x00\x00\x02'
CMD_BYPASS_AUTO = b'\x85\x15\x02\x01'
CMD_SENSOR_TEMP_OFF = b'\x03\x1d\x01\x04\x00'
CMD_SENSOR_TEMP_AUTO = b'\x03\x1d\x01\x04\x01'
CMD_SENSOR_TEMP_ON = b'\x03\x1d\x01\x04\x02'
CMD_SENSOR_HUMC_OFF = b'\x03\x1d\x01\x06\x00'
CMD_SENSOR_HUMC_AUTO = b'\x03\x1d\x01\x06\x01'
CMD_SENSOR_HUMC_ON = b'\x03\x1d\x01\x06\x02'
CMD_SENSOR_HUMP_OFF = b'\x03\x1d\x01\x07\x00'
CMD_SENSOR_HUMP_AUTO = b'\x03\x1d\x01\x07\x01'
CMD_SENSOR_HUMP_ON = b'\x03\x1d\x01\x07\x02'
CMD_BOOST_MODE_1                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x08\x07\x00\x00\x03'   # Partymode 0.5h
CMD_BOOST_MODE_2                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x10\x0E\x00\x00\x03'   # Partymode 1h
CMD_BOOST_MODE_3                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x20\x1C\x00\x00\x03'   # Partymode 2h
CMD_BOOST_MODE_4                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x30\x2A\x00\x00\x03'   # Partymode 3h
CMD_BOOST_MODE_5                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x40\x38\x00\x00\x03'   # Partymode 4h
CMD_BOOST_MODE_6                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x50\x46\x00\x00\x03'   # Partymode 5h
CMD_BOOST_MODE_7                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x60\x54\x00\x00\x03'   # Partymode 6h
CMD_BOOST_MODE_8                = b'\x84\x15\x01\x06\x00\x00\x00\x00\x80\x70\x00\x00\x03'   # Partymode 8h
CMD_BOOST_MODE_0                = b'\x85\x15\x01\x06' 

# Sensor locations
SENSOR_AWAY = 16
SENSOR_OPERATING_MODE_BIS = 49
SENSOR_OPERATING_MODE = 56
SENSOR_FAN_SPEED_MODE = 65
SENSOR_BYPASS_ACTIVATIONSTATE = 66
SENSOR_PROFILE_TEMPERATURE = 67
SENSOR_FAN_MODE_SUPPLY = 70
SENSOR_FAN_MODE_EXHAUST = 71
SENSOR_FAN_NEXT_CHANGE = 81
SENSOR_BYPASS_NEXT_CHANGE = 82
SENSOR_SUPPLY_NEXT_CHANGE = 86
SENSOR_EXHAUST_NEXT_CHANGE = 87
SENSOR_FAN_EXHAUST_DUTY = 117
SENSOR_FAN_SUPPLY_DUTY = 118
SENSOR_FAN_EXHAUST_FLOW = 119
SENSOR_FAN_SUPPLY_FLOW = 120
SENSOR_FAN_EXHAUST_SPEED = 121
SENSOR_FAN_SUPPLY_SPEED = 122
SENSOR_POWER_CURRENT = 128
SENSOR_POWER_TOTAL_YEAR = 129
SENSOR_POWER_TOTAL = 130
SENSOR_PREHEATER_POWER_TOTAL_YEAR = 144
SENSOR_PREHEATER_POWER_TOTAL = 145
SENSOR_PREHEATER_POWER_CURRENT = 146
SENSOR_SETTING_RF_PAIRING = 176
SENSOR_DAYS_TO_REPLACE_FILTER = 192
SENSOR_CURRENT_RMOT = 209
SENSOR_HEATING_SEASON = 210
SENSOR_COOLING_SEASON = 211
SENSOR_TARGET_TEMPERATURE = 212
SENSOR_AVOIDED_HEATING_CURRENT = 213
SENSOR_AVOIDED_HEATING_TOTAL_YEAR = 214
SENSOR_AVOIDED_HEATING_TOTAL = 215
SENSOR_AVOIDED_COOLING_CURRENT = 216
SENSOR_AVOIDED_COOLING_TOTAL_YEAR = 217
SENSOR_AVOIDED_COOLING_TOTAL = 218
SENSOR_AVOIDED_COOLING_CURRENT_Target = 219
SENSOR_TEMPERATURE_SUPPLY = 221
SENSOR_COMFORTCONTROL_MODE = 225
SENSOR_BYPASS_STATE = 227
SENSOR_FROSTPROTECTION_UNBALANCE = 228
SENSOR_TEMPERATURE_EXTRACT = 274
SENSOR_TEMPERATURE_EXHAUST = 275
SENSOR_TEMPERATURE_OUTDOOR = 276
SENSOR_TEMPERATURE_AFTER_PREHEATER = 277
SENSOR_HUMIDITY_EXTRACT = 290
SENSOR_HUMIDITY_EXHAUST = 291
SENSOR_HUMIDITY_OUTDOOR = 292
SENSOR_HUMIDITY_AFTER_PREHEATER = 293
SENSOR_HUMIDITY_SUPPLY = 294



import logging
import select
import socket

from .message import *

_LOGGER = logging.getLogger('bridge')


class Bridge(object):
    """Implements an interface to send and receive messages from the Bridge."""

    PORT = 56747

    @staticmethod
    def discover(host=None, timeout=5):
        """Broadcast the network and look for local bridges."""

        # Setup socket
        udpsocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        udpsocket.setblocking(0)
        udpsocket.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

        # Send broadcast packet
        if host is None:
            udpsocket.sendto(b"\x0a\x00", ('<broadcast>', Bridge.PORT))
        else:
            udpsocket.sendto(b"\x0a\x00", (host, Bridge.PORT))

        # Try to read response
        parser = DiscoveryOperation()
        bridges = []
        while True:
            ready = select.select([udpsocket], [], [], timeout)
            if not ready[0]:
                break

            data, source = udpsocket.recvfrom(100)

            # Parse data
            parser.ParseFromString(data)
            ip_address = parser.searchGatewayResponse.ipaddress
            uuid = parser.searchGatewayResponse.uuid

            # Add a new Bridge to the list
            bridges.append(
                Bridge(ip_address, uuid)
            )

            # Don't look for other bridges if we directly discovered it by IP
            if host:
                break

        udpsocket.close()

        # Return found bridges
        return bridges

    def __init__(self, host: str, uuid: str) -> None:
        self.host = host
        self.uuid = uuid

        self._socket = None
        self.debug = False

    def connect(self) -> bool:
        """Open connection to the bridge."""

        if self._socket is None:
            tcpsocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            tcpsocket.connect((self.host, Bridge.PORT))
            tcpsocket.setblocking(0)
            self._socket = tcpsocket

        return True

    def disconnect(self) -> bool:
        """Close connection to the bridge."""

        self._socket.close()
        self._socket = None

        return True

    def is_connected(self):
        """Returns weather there is an open socket."""

        return self._socket is not None

    def read_message(self, timeout=1) -> Message:
        """Read a message from the connection."""

        if self._socket is None:
            raise BrokenPipeError()

        # Check if there is data available
        ready = select.select([self._socket], [], [], timeout)
        if not ready[0]:
            # Timeout
            return None

        # Read packet size
        msg_len_buf = self._socket.recv(4)
        if not msg_len_buf:
            # No data, but there has to be.
            raise BrokenPipeError()

        # Read rest of packet
        msg_len = struct.unpack('>L', msg_len_buf)[0]
        msg_buf = self._socket.recv(msg_len)
        if not msg_buf:
            # No data, but there has to be.
            raise BrokenPipeError()

        # Decode message
        message = Message.decode(msg_len_buf + msg_buf)

        # Debug message
        _LOGGER.debug("RX %s", message)

        return message

    def write_message(self, message: Message) -> bool:
        """Send a message."""

        if self._socket is None:
            raise Exception('Not connected!')

        # Construct packet
        packet = message.encode()

        # Debug message
        _LOGGER.debug("TX %s", message)

        # Send packet
        try:
            self._socket.sendall(packet)
        except BrokenPipeError:
            self.disconnect()
            return False

        return True



class PyComfoConnectError(Exception):
    """ Base error for PyComfoConnect """
    pass


class PyComfoConnectBadRequest(PyComfoConnectError):
    """ An error occured because the request was invalid. """
    pass


class PyComfoConnectInternalError(PyComfoConnectError):
    """ An error occured because something went wrong inside the bridge. """
    pass


class PyComfoConnectNotReachable(PyComfoConnectError):
    """ An error occured because the bridge could not reach the ventilation unit. """
    pass


class PyComfoConnectOtherSession(PyComfoConnectError):
    """ An error occured because the bridge is already connected to a different device. """

    def __init__(self, devicename):
        self.devicename = devicename


class PyComfoConnectNotAllowed(PyComfoConnectError):
    """ An error occured because you have not authenticated yet. """
    pass


class PyComfoConnectNoResources(PyComfoConnectError):
    pass


class PyComfoConnectNotExist(PyComfoConnectError):
    pass


class PyComfoConnectRmiError(PyComfoConnectError):
    pass



import logging
import queue
import struct
import threading
import time

from .bridge import Bridge
from .error import *
from .message import Message
from .zehnder_pb2 import *

KEEPALIVE = 60

DEFAULT_LOCAL_UUID = bytes.fromhex('00000000000000000000000000001337')
DEFAULT_LOCAL_DEVICENAME = 'pycomfoconnect'
DEFAULT_PIN = 0

_LOGGER = logging.getLogger('comfoconnect')

# Sensor variable size
RPDO_TYPE_MAP = {
    16: 1,
    33: 1,
    37: 1,
    49: 1,
    53: 1,
    56: 1,
    65: 1,
    66: 1,
    67: 1,
    70: 1,
    71: 1,
    81: 3,
    82: 3,
    85: 3,
    86: 3,
    87: 3,
    117: 1,
    118: 1,
    119: 2,
    120: 2,
    121: 2,
    122: 2,
    128: 2,
    129: 2,
    130: 2,
    144: 2,
    145: 2,
    146: 2,
    176: 1,
    192: 2,
    208: 1,
    209: 6,
    210: 0,
    211: 0,
    212: 6,
    213: 2,
    214: 2,
    215: 2,
    216: 2,
    217: 2,
    218: 2,
    219: 2,
    221: 6,
    224: 1,
    225: 1,
    226: 2,
    227: 1,
    228: 1,
    274: 6,
    275: 6,
    276: 6,
    277: 6,
    290: 1,
    291: 1,
    292: 1,
    293: 1,
    294: 1,
    321: 2,
    325: 2,
    337: 3,
    338: 3,
    341: 3,
    369: 1,
    370: 1,
    371: 1,
    372: 1,
    384: 6,
    386: 0,
    400: 6,
    401: 1,
    402: 0,
    416: 6,
    417: 6,
    418: 1,
    419: 0,
}

# Product ID Map
PRODUCT_ID_MAP = {
    1: "ComfoAirQ",
    2: "ComfoSense",
    3: "ComfoSwitch",
    4: "OptionBox",
    5: "ZehnderGateway",
    6: "ComfoCool",
    7: "KNXGateway",
    8: "Service Tool",
    9: "Production test tool",
    10: "Design verification test tool"
}

class ComfoConnect(object):
    """Implements the commands to communicate with the ComfoConnect ventilation unit."""

    """Callback function to invoke when sensor updates are received."""
    callback_sensor = None

    def __init__(self, bridge: Bridge, local_uuid=DEFAULT_LOCAL_UUID, local_devicename=DEFAULT_LOCAL_DEVICENAME,
                 pin=DEFAULT_PIN):
        self._bridge = bridge
        self._local_uuid = local_uuid
        self._local_devicename = local_devicename
        self._pin = pin
        self._reference = 1

        self._queue = queue.Queue()
        self._connected = threading.Event()
        self._stopping = False
        self._message_thread = None
        self._connection_thread = None

        self.sensors = {}

    # ==================================================================================================================
    # Core functions
    # ==================================================================================================================

    def connect(self, takeover=False):
        """Connect to the bridge and login. Disconnect existing clients if needed by default."""

        try:
            # Start connection
            self._connect(takeover=takeover)

        except PyComfoConnectNotAllowed:
            raise Exception('Could not connect to the bridge since the PIN seems to be invalid.')

        except PyComfoConnectOtherSession:
            raise Exception('Could not connect to the bridge since there is already an open session.')

        except Exception as exc:
            _LOGGER.error(exc)
            raise Exception('Could not connect to the bridge.')

        # Set the stopping flag
        self._stopping = False
        self._connected.clear()

        # Start connection thread
        self._connection_thread = threading.Thread(target=self._connection_thread_loop)
        self._connection_thread.start()

        if not self._connected.wait(10):
            raise Exception('Could not connect to bridge since it didn\'t reply on time.')

        return True

    def disconnect(self):
        """Disconnect from the bridge."""

        # Set the stopping flag
        self._stopping = True

        # Wait for the background thread to finish
        self._connection_thread.join()
        self._connection_thread = None

    def is_connected(self):
        """Returns whether there is a connection with the bridge."""

        return self._bridge.is_connected()

    def register_sensor(self, sensor_id: int, sensor_type: int = None):
        """Register a sensor on the bridge and keep it in memory that we are registered to this sensor."""

        if not sensor_type:
            sensor_type = RPDO_TYPE_MAP.get(sensor_id)

        if sensor_type is None:
            raise Exception("Registering sensor %d with unknown type" % sensor_id)

        # Register on bridge
        try:
            reply = self.cmd_rpdo_request(sensor_id, sensor_type)

        except PyComfoConnectNotAllowed:
            return None

        # Register in memory
        self.sensors[sensor_id] = sensor_type

        return reply

    def unregister_sensor(self, sensor_id: int, sensor_type: int = None):
        """Register a sensor on the bridge and keep it in memory that we are registered to this sensor."""

        if sensor_type is None:
            sensor_type = RPDO_TYPE_MAP.get(sensor_id)

        if sensor_type is None:
            raise Exception("Unregistering sensor %d with unknown type" % sensor_id)

        # Unregister in memory
        self.sensors.pop(sensor_id, None)

        # Unregister on bridge
        self.cmd_rpdo_request(sensor_id, sensor_type, timeout=0)

    def _command(self, command, params=None, use_queue=True):
        """Sends a command and wait for a response if the request is known to return a result."""

        # Construct the message
        message = Message.create(
            self._local_uuid,
            self._bridge.uuid,
            command,
            {'reference': self._reference},
            params
        )

        # Increase message reference
        self._reference += 1

        # Send the message
        self._bridge.write_message(message)

        try:
            # Check if this command has a confirm type set
            confirm_type = message.class_to_confirm[command]

            # Read a message
            reply = self._get_reply(confirm_type, use_queue=use_queue)

            return reply

        except KeyError:
            return None

    def _get_reply(self, confirm_type=None, timeout=5, use_queue=True):
        """Pops a message of the queue, optionally looking for a specific type."""

        start = time.time()

        while True:
            message = None

            if use_queue:
                try:
                    # Fetch the message from the queue.  The network thread has put it there for us.
                    message = self._queue.get(timeout=timeout)
                    if message:
                        self._queue.task_done()
                except queue.Empty:
                    # We got no message
                    pass

            else:
                # Fetch the message directly from the socket
                message = self._bridge.read_message(timeout=timeout)

            if message:
                # Check status code
                if message.cmd.result == GatewayOperation.OK:
                    pass
                elif message.cmd.result == GatewayOperation.BAD_REQUEST:
                    raise PyComfoConnectBadRequest()
                elif message.cmd.result == GatewayOperation.INTERNAL_ERROR:
                    raise PyComfoConnectInternalError()
                elif message.cmd.result == GatewayOperation.NOT_REACHABLE:
                    raise PyComfoConnectNotReachable()
                elif message.cmd.result == GatewayOperation.OTHER_SESSION:
                    raise PyComfoConnectOtherSession(message.msg.devicename)
                elif message.cmd.result == GatewayOperation.NOT_ALLOWED:
                    raise PyComfoConnectNotAllowed()
                elif message.cmd.result == GatewayOperation.NO_RESOURCES:
                    raise PyComfoConnectNoResources()
                elif message.cmd.result == GatewayOperation.NOT_EXIST:
                    raise PyComfoConnectNotExist()
                elif message.cmd.result == GatewayOperation.RMI_ERROR:
                    raise PyComfoConnectRmiError()

                if confirm_type is None:
                    # We just need a message
                    return message
                elif message.msg.__class__ == confirm_type:
                    # We need the message with the correct type
                    return message
                else:
                    # We got a message with an incorrect type. Hopefully, this doesn't happen to often,
                    # since we just put it back on the queue.
                    self._queue.put(message)

            if time.time() - start > timeout:
                raise ValueError('Timeout waiting for response.')

    # ==================================================================================================================
    # Connection thread
    # ==================================================================================================================
    def _connection_thread_loop(self):
        """Makes sure that there is a connection open."""

        self._stopping = False
        while not self._stopping:

            # Start connection
            if not self.is_connected():

                # Wait a bit to avoid hammering the bridge
                time.sleep(5)

                try:
                    # Connect or re-connect
                    self._connect()

                except PyComfoConnectOtherSession:
                    self._bridge.disconnect()
                    _LOGGER.error('Could not connect to the bridge since there is already an open session.')
                    continue

                except Exception as exc:
                    _LOGGER.error(exc)
                    raise Exception('Could not connect to the bridge.')

            # Start background thread
            self._message_thread = threading.Thread(target=self._message_thread_loop)
            self._message_thread.start()

            # Re-register for sensor updates
            for sensor_id in self.sensors:
                self.cmd_rpdo_request(sensor_id, self.sensors[sensor_id])

            # Send the event that we are ready
            self._connected.set()

            # Wait until the message thread stops working
            self._message_thread.join()

            # Close socket connection
            self._bridge.disconnect()

    def _connect(self, takeover=False):
        """Connect to the bridge and login. Disconnect existing clients if needed by default."""

        try:
            # Connect to the bridge
            self._bridge.connect()

            # Login
            self.cmd_start_session(takeover, use_queue=False)

        except PyComfoConnectNotAllowed:
            # No dice, maybe we are not registered yet...

            # Register
            self.cmd_register_app(self._local_uuid, self._local_devicename, self._pin, use_queue=False)

            # Login
            self.cmd_start_session(takeover, use_queue=False)

        return True

    # ==================================================================================================================
    # Message thread
    # ==================================================================================================================

    def _message_thread_loop(self):
        """Listen for incoming messages and queue them or send them to a callback method."""

        # Reinitialise the queues
        self._queue = queue.Queue()

        next_keepalive = 0

        while not self._stopping:

            # Sends a keepalive every KEEPALIVE seconds.
            if time.time() > next_keepalive:
                next_keepalive = time.time() + KEEPALIVE
                self.cmd_keepalive()

            try:
                # Read a message from the bridge.
                message = self._bridge.read_message()

            except BrokenPipeError as exc:
                # Close this thread. The connection_thread will restart us.
                _LOGGER.warning('The connection was broken. We will try to reconnect later.')
                return

            if message:
                if message.cmd.type == GatewayOperation.CnRpdoNotificationType:
                    self._handle_rpdo_notification(message)

                elif message.cmd.type == GatewayOperation.GatewayNotificationType:
                    _LOGGER.info('Unhandled GatewayNotificationType')
                    # TODO: We should probably handle these somehow
                    pass

                elif message.cmd.type == GatewayOperation.CnNodeNotificationType:
                    _LOGGER.info('CnNodeNotificationType: %s @ Node Id %d [%s]', 
                        PRODUCT_ID_MAP[message.msg.productId], 
                        message.msg.nodeId, 
                        message.msg.NodeModeType.Name(message.msg.mode))
                    # TODO: We should probably handle these somehow
                    pass

                elif message.cmd.type == GatewayOperation.CnAlarmNotificationType:
                    _LOGGER.info('Unhandled CnAlarmNotificationType')
                    # TODO: We should probably handle these somehow
                    pass

                elif message.cmd.type == GatewayOperation.CloseSessionRequestType:
                    _LOGGER.info('The Bridge has asked us to close the connection. We will try to reconnect later.')
                    # Close this thread. The connection_thread will restart us.
                    return

                else:
                    # Send other messages to a queue
                    self._queue.put(message)

        return

    def _handle_rpdo_notification(self, message):
        """Update internal sensor state and invoke callback."""

        # Only process CnRpdoNotificationType
        if message.cmd.type != GatewayOperation.CnRpdoNotificationType:
            return False

        # Extract data
        data = message.msg.data.hex()
        if len(data) == 2:
            val = struct.unpack('b', message.msg.data)[0]
        elif len(data) == 4:
            val = struct.unpack('h', message.msg.data)[0]
        elif len(data) == 8:
            val = data
        else:
            val = data

        # Update local state
        # self.sensors[message.msg.pdid] = val

        if self.callback_sensor:
            self.callback_sensor(message.msg.pdid, val)

        return True

    # ==================================================================================================================
    # Commands
    # ==================================================================================================================

    def cmd_start_session(self, take_over=False, use_queue: bool = True):
        """Starts the session on the device by logging in and optionally disconnecting an already existing session."""

        reply = self._command(
            StartSessionRequest,
            {
                'takeover': take_over
            },
            use_queue=use_queue
        )
        return reply  # TODO: parse output

    def cmd_close_session(self, use_queue: bool = True):
        """Stops the current session."""

        reply = self._command(
            CloseSessionRequest,
            use_queue=use_queue
        )
        return reply  # TODO: parse output

    def cmd_list_registered_apps(self, use_queue: bool = True):
        """Returns a list of all the registered clients."""

        reply = self._command(
            ListRegisteredAppsRequest,
            use_queue=use_queue
        )
        return [
            {'uuid': app.uuid, 'devicename': app.devicename} for app in reply.msg.apps
        ]

    def cmd_register_app(self, uuid, device_name, pin, use_queue: bool = True):
        """Register a new app by specifying our own uuid, device_name and pin code."""

        reply = self._command(
            RegisterAppRequest,
            {
                'uuid': uuid,
                'devicename': device_name,
                'pin': pin,
            },
            use_queue=use_queue
        )
        return reply  # TODO: parse output

    def cmd_deregister_app(self, uuid, use_queue: bool = True):
        """Remove the specified app from the registration list."""

        if uuid == self._local_uuid:
            raise Exception('You should not deregister yourself.')

        try:
            self._command(
                DeregisterAppRequest,
                {
                    'uuid': uuid
                },
                use_queue=use_queue
            )
            return True

        except PyComfoConnectBadRequest:
            return False

    def cmd_version_request(self, use_queue: bool = True):
        """Returns version information."""

        reply = self._command(
            VersionRequest,
            use_queue=use_queue
        )
        return {
            'gatewayVersion': reply.msg.gatewayVersion,
            'serialNumber': reply.msg.serialNumber,
            'comfoNetVersion': reply.msg.comfoNetVersion,
        }

    def cmd_time_request(self, use_queue: bool = True):
        """Returns the current time on the device."""

        reply = self._command(
            CnTimeRequest,
            use_queue=use_queue
        )
        return reply.msg.currentTime

    def cmd_rmi_request(self, message, node_id: int = 1, use_queue: bool = True):
        """Sends a RMI request."""

        reply = self._command(
            CnRmiRequest,
            {
                'nodeId': node_id or 1,
                'message': message
            },
            use_queue=use_queue
        )
        return True

    def cmd_rpdo_request(self, pdid: int, type: int = 1, zone: int = 1, timeout=None, use_queue: bool = True):
        """Register a RPDO request."""

        reply = self._command(
            CnRpdoRequest,
            {
                'pdid': pdid,
                'type': type,
                'zone': zone or 1,
                'timeout': timeout
            },
            use_queue=use_queue
        )
        return reply

    def cmd_keepalive(self, use_queue: bool = True):
        """Sends a keepalive."""

        self._command(
            KeepAlive,
            use_queue=use_queue
        )
        return True



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

long_description = open('README.rst').read()

setup(
    name='pycomfoconnect',
    version='0.5.1',
    license='MIT',
    url='https://github.com/michaelarnauts/comfoconnect',
    author='Michaël Arnauts',
    author_email='michael.arnauts@gmail.com',
    description='Python interface for the Zehnder ComfoConnect LAN C bridge.',
    long_description=long_description,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=list(val.strip() for val in open('requirements.txt')),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ]
)



"""
PyComfoConnect: Manage your Zehnder ComfoConnect Q350/Q450/Q650 ventilation unit
"""

DEFAULT_UUID = '00000000000000000000000000000001'
DEFAULT_NAME = 'pycomfoconnect'
DEFAULT_PIN = 0

__author__ = 'Michaël Arnauts <michael.arnauts@gmail.com>'

from .comfoconnect import *
from .error import *
from .const import *


#!/usr/bin/python3
import argparse
from time import sleep

from pycomfoconnect import *

parser = argparse.ArgumentParser()
parser.add_argument('--ip', help='ip address of the bridge')
args = parser.parse_args()

## Configuration #######################################################################################################

pin = 0
local_name = 'Computer'
local_uuid = bytes.fromhex('00000000000000000000000000000005')


def bridge_discovery():
    ## Bridge discovery ################################################################################################

    # Method 1: Use discovery to initialise Bridge
    # bridges = Bridge.discover(timeout=1)
    # if bridges:
    #     bridge = bridges[0]
    # else:
    #     bridge = None

    # Method 2: Use direct discovery to initialise Bridge
    bridges = Bridge.discover(args.ip)
    if bridges:
        bridge = bridges[0]
    else:
        bridge = None

    # Method 3: Setup bridge manually
    # bridge = Bridge(args.ip, bytes.fromhex('0000000000251010800170b3d54264b4'))

    if bridge is None:
        print("No bridges found!")
        exit(1)

    print("Bridge found: %s (%s)" % (bridge.uuid.hex(), bridge.host))
    bridge.debug = True

    return bridge


def callback_sensor(var, value):
    ## Callback sensors ################################################################################################

    print("%s = %s" % (var, value))


def main():
    # Discover the bridge
    bridge = bridge_discovery()

    ## Setup a Comfoconnect session  ###################################################################################

    comfoconnect = ComfoConnect(bridge, local_uuid, local_name, pin)
    comfoconnect.callback_sensor = callback_sensor

    try:
        # Connect to the bridge
        # comfoconnect.connect(False)  # Don't disconnect existing clients.
        comfoconnect.connect(True)  # Disconnect existing clients.

    except Exception as e:
        print('ERROR: %s' % e)
        exit(1)

    ## Register sensors ################################################################################################

    # comfoconnect.register_sensor(SENSOR_FAN_NEXT_CHANGE)  # General: Countdown until next fan speed change
    # comfoconnect.register_sensor(SENSOR_FAN_SPEED_MODE)  # Fans: Fan speed setting
    # comfoconnect.register_sensor(SENSOR_FAN_SUPPLY_DUTY)  # Fans: Supply fan duty
    # comfoconnect.register_sensor(SENSOR_FAN_EXHAUST_DUTY)  # Fans: Exhaust fan duty
    # comfoconnect.register_sensor(SENSOR_FAN_SUPPLY_FLOW)  # Fans: Supply fan flow
    # comfoconnect.register_sensor(SENSOR_FAN_EXHAUST_FLOW)  # Fans: Exhaust fan flow
    # comfoconnect.register_sensor(SENSOR_FAN_SUPPLY_SPEED)  # Fans: Supply fan speed
    # comfoconnect.register_sensor(SENSOR_FAN_EXHAUST_SPEED)  # Fans: Exhaust fan speed
    # comfoconnect.register_sensor(SENSOR_POWER_CURRENT)  # Power Consumption: Current Ventilation
    # comfoconnect.register_sensor(SENSOR_POWER_TOTAL_YEAR)  # Power Consumption: Total year-to-date
    # comfoconnect.register_sensor(SENSOR_POWER_TOTAL)  # Power Consumption: Total from start
    # comfoconnect.register_sensor(SENSOR_DAYS_TO_REPLACE_FILTER)  # Days left before filters must be replaced
    # comfoconnect.register_sensor(SENSOR_AVOIDED_HEATING_CURRENT)  # Avoided Heating: Avoided actual
    # comfoconnect.register_sensor(SENSOR_AVOIDED_HEATING_TOTAL_YEAR)  # Avoided Heating: Avoided year-to-date
    # comfoconnect.register_sensor(SENSOR_AVOIDED_HEATING_TOTAL)  # Avoided Heating: Avoided total
    comfoconnect.register_sensor(SENSOR_TEMPERATURE_SUPPLY)  # Temperature & Humidity: Supply Air (temperature)
    comfoconnect.register_sensor(SENSOR_TEMPERATURE_EXTRACT)  # Temperature & Humidity: Extract Air (temperature)
    comfoconnect.register_sensor(SENSOR_TEMPERATURE_EXHAUST)  # Temperature & Humidity: Exhaust Air (temperature)
    comfoconnect.register_sensor(SENSOR_TEMPERATURE_OUTDOOR)  # Temperature & Humidity: Outdoor Air (temperature)
    comfoconnect.register_sensor(SENSOR_HUMIDITY_SUPPLY)  # Temperature & Humidity: Supply Air (temperature)
    comfoconnect.register_sensor(SENSOR_HUMIDITY_EXTRACT)  # Temperature & Humidity: Extract Air (temperature)
    comfoconnect.register_sensor(SENSOR_HUMIDITY_EXHAUST)  # Temperature & Humidity: Exhaust Air (temperature)
    comfoconnect.register_sensor(SENSOR_HUMIDITY_OUTDOOR)  # Temperature & Humidity: Outdoor Air (temperature)
    comfoconnect.register_sensor(SENSOR_BYPASS_STATE)  # Bypass state
    comfoconnect.register_sensor(SENSOR_OPERATING_MODE)  # Operating mode
    comfoconnect.register_sensor(SENSOR_OPERATING_MODE_BIS)  # Operating mode (bis)

    # comfoconnect.register_sensor(16) # 1
    # comfoconnect.register_sensor(33) # 1
    # comfoconnect.register_sensor(37) # 0
    # comfoconnect.register_sensor(53) # -1
    # comfoconnect.register_sensor(66) # 0
    # comfoconnect.register_sensor(67) # 0
    # comfoconnect.register_sensor(70) # 0
    # comfoconnect.register_sensor(71) # 0
    # comfoconnect.register_sensor(82) # ffffffff
    # comfoconnect.register_sensor(85) # ffffffff
    # comfoconnect.register_sensor(86) # ffffffff
    # comfoconnect.register_sensor(87) # ffffffff
    # comfoconnect.register_sensor(144) # 0
    # comfoconnect.register_sensor(145) # 0
    # comfoconnect.register_sensor(146) # 0
    # comfoconnect.register_sensor(176) # 0
    # comfoconnect.register_sensor(208) # 0
    # comfoconnect.register_sensor(210) # 0
    # comfoconnect.register_sensor(211) # 0
    # comfoconnect.register_sensor(212) # 228
    # comfoconnect.register_sensor(216) # 0
    # comfoconnect.register_sensor(217) # 28
    # comfoconnect.register_sensor(218) # 28
    # comfoconnect.register_sensor(219) # 0
    # comfoconnect.register_sensor(224) # 3
    # comfoconnect.register_sensor(225) # 1
    # comfoconnect.register_sensor(226) # 100
    # comfoconnect.register_sensor(228) # 0
    # comfoconnect.register_sensor(321) # 15
    # comfoconnect.register_sensor(325) # 1
    # comfoconnect.register_sensor(337) #
    # comfoconnect.register_sensor(338) # 00000000
    # comfoconnect.register_sensor(341) # 00000000
    # comfoconnect.register_sensor(369) # 0
    # comfoconnect.register_sensor(370) # 0
    # comfoconnect.register_sensor(371) # 0
    # comfoconnect.register_sensor(372) # 0
    # comfoconnect.register_sensor(384) # 0
    # comfoconnect.register_sensor(386) # 0
    # comfoconnect.register_sensor(400) # 0
    # comfoconnect.register_sensor(401) # 0
    # comfoconnect.register_sensor(402) # 0
    # comfoconnect.register_sensor(416) # -400
    # comfoconnect.register_sensor(417) # 100
    # comfoconnect.register_sensor(418) # 0
    # comfoconnect.register_sensor(419) # 0

    ## Execute functions ###############################################################################################

    # ListRegisteredApps
    # for app in comfoconnect.cmd_list_registered_apps():
    #     print('%s: %s' % (app['uuid'].hex(), app['devicename']))

    # DeregisterApp
    # comfoconnect.cmd_deregister_app(bytes.fromhex('00000000000000000000000000000001'))

    # VersionRequest
    version = comfoconnect.cmd_version_request()
    print(version)

    # TimeRequest
    # timeinfo = comfoconnect.cmd_time_request()
    # print(timeinfo)

    ## Executing functions #############################################################################################

    # comfoconnect.cmd_rmi_request(CMD_FAN_MODE_AWAY)  # Go to away mode
    # comfoconnect.cmd_rmi_request(CMD_FAN_MODE_LOW)  # Set fan speed to 1
    # comfoconnect.cmd_rmi_request(CMD_FAN_MODE_MEDIUM)  # Set fan speed to 2
    # comfoconnect.cmd_rmi_request(CMD_FAN_MODE_HIGH)  # Set fan speed to 3

    ## Example interaction #############################################################################################

    try:
        print('Waiting... Stop with CTRL+C')
        while True:
            # Callback messages will arrive in the callback method.
            sleep(1)

            if not comfoconnect.is_connected():
                print('We are not connected anymore...')

    except KeyboardInterrupt:
        pass

    ## Closing the session #############################################################################################

    print('Disconnecting...')
    comfoconnect.disconnect()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: zehnder.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rzehnder.proto\"\x80\x01\n\x12\x44iscoveryOperation\x12\x33\n\x14searchGatewayRequest\x18\x01 \x01(\x0b\x32\x15.SearchGatewayRequest\x12\x35\n\x15searchGatewayResponse\x18\x02 \x01(\x0b\x32\x16.SearchGatewayResponse\"\x16\n\x14SearchGatewayRequest\"I\n\x15SearchGatewayResponse\x12\x11\n\tipaddress\x18\x01 \x02(\t\x12\x0c\n\x04uuid\x18\x02 \x02(\x0c\x12\x0f\n\x07version\x18\x03 \x02(\r\"\xde\x10\n\x10GatewayOperation\x12-\n\x04type\x18\x01 \x01(\x0e\x32\x1f.GatewayOperation.OperationType\x12/\n\x06result\x18\x02 \x01(\x0e\x32\x1f.GatewayOperation.GatewayResult\x12\x19\n\x11resultDescription\x18\x03 \x01(\t\x12\x11\n\treference\x18\x04 \x01(\r\"\x95\x0e\n\rOperationType\x12\x0f\n\x0bNoOperation\x10\x00\x12\x19\n\x15SetAddressRequestType\x10\x01\x12\x1a\n\x16RegisterAppRequestType\x10\x02\x12\x1b\n\x17StartSessionRequestType\x10\x03\x12\x1b\n\x17\x43loseSessionRequestType\x10\x04\x12!\n\x1dListRegisteredAppsRequestType\x10\x05\x12\x1c\n\x18\x44\x65registerAppRequestType\x10\x06\x12\x18\n\x14\x43hangePinRequestType\x10\x07\x12 \n\x1cGetRemoteAccessIdRequestType\x10\x08\x12 \n\x1cSetRemoteAccessIdRequestType\x10\t\x12\x1b\n\x17GetSupportIdRequestType\x10\n\x12\x1b\n\x17SetSupportIdRequestType\x10\x0b\x12\x17\n\x13GetWebIdRequestType\x10\x0c\x12\x17\n\x13SetWebIdRequestType\x10\r\x12\x18\n\x14SetPushIdRequestType\x10\x0e\x12\x14\n\x10\x44\x65\x62ugRequestType\x10\x0f\x12\x16\n\x12UpgradeRequestType\x10\x10\x12 \n\x1cSetDeviceSettingsRequestType\x10\x11\x12\x16\n\x12VersionRequestType\x10\x12\x12\x19\n\x15SetAddressConfirmType\x10\x33\x12\x1a\n\x16RegisterAppConfirmType\x10\x34\x12\x1b\n\x17StartSessionConfirmType\x10\x35\x12\x1b\n\x17\x43loseSessionConfirmType\x10\x36\x12!\n\x1dListRegisteredAppsConfirmType\x10\x37\x12\x1c\n\x18\x44\x65registerAppConfirmType\x10\x38\x12\x18\n\x14\x43hangePinConfirmType\x10\x39\x12 \n\x1cGetRemoteAccessIdConfirmType\x10:\x12 \n\x1cSetRemoteAccessIdConfirmType\x10;\x12\x1b\n\x17GetSupportIdConfirmType\x10<\x12\x1b\n\x17SetSupportIdConfirmType\x10=\x12\x17\n\x13GetWebIdConfirmType\x10>\x12\x17\n\x13SetWebIdConfirmType\x10?\x12\x18\n\x14SetPushIdConfirmType\x10@\x12\x14\n\x10\x44\x65\x62ugConfirmType\x10\x41\x12\x16\n\x12UpgradeConfirmType\x10\x42\x12 \n\x1cSetDeviceSettingsConfirmType\x10\x43\x12\x16\n\x12VersionConfirmType\x10\x44\x12\x1b\n\x17GatewayNotificationType\x10\x64\x12\x11\n\rKeepAliveType\x10\x65\x12\x14\n\x10\x46\x61\x63toryResetType\x10\x66\x12\x15\n\x11\x43nTimeRequestType\x10\x1e\x12\x15\n\x11\x43nTimeConfirmType\x10\x1f\x12\x15\n\x11\x43nNodeRequestType\x10*\x12\x1a\n\x16\x43nNodeNotificationType\x10 \x12\x14\n\x10\x43nRmiRequestType\x10!\x12\x15\n\x11\x43nRmiResponseType\x10\"\x12\x19\n\x15\x43nRmiAsyncRequestType\x10#\x12\x19\n\x15\x43nRmiAsyncConfirmType\x10$\x12\x1a\n\x16\x43nRmiAsyncResponseType\x10%\x12\x15\n\x11\x43nRpdoRequestType\x10&\x12\x15\n\x11\x43nRpdoConfirmType\x10\'\x12\x1a\n\x16\x43nRpdoNotificationType\x10(\x12\x1b\n\x17\x43nAlarmNotificationType\x10)\x12 \n\x1c\x43nFupReadRegisterRequestType\x10\x46\x12 \n\x1c\x43nFupReadRegisterConfirmType\x10G\x12 \n\x1c\x43nFupProgramBeginRequestType\x10H\x12 \n\x1c\x43nFupProgramBeginConfirmType\x10I\x12\x1b\n\x17\x43nFupProgramRequestType\x10J\x12\x1b\n\x17\x43nFupProgramConfirmType\x10K\x12\x1e\n\x1a\x43nFupProgramEndRequestType\x10L\x12\x1e\n\x1a\x43nFupProgramEndConfirmType\x10M\x12\x18\n\x14\x43nFupReadRequestType\x10N\x12\x18\n\x14\x43nFupReadConfirmType\x10O\x12\x19\n\x15\x43nFupResetRequestType\x10P\x12\x19\n\x15\x43nFupResetConfirmType\x10Q\"\xa3\x01\n\rGatewayResult\x12\x06\n\x02OK\x10\x00\x12\x0f\n\x0b\x42\x41\x44_REQUEST\x10\x01\x12\x12\n\x0eINTERNAL_ERROR\x10\x02\x12\x11\n\rNOT_REACHABLE\x10\x03\x12\x11\n\rOTHER_SESSION\x10\x04\x12\x0f\n\x0bNOT_ALLOWED\x10\x05\x12\x10\n\x0cNO_RESOURCES\x10\x06\x12\r\n\tNOT_EXIST\x10\x07\x12\r\n\tRMI_ERROR\x10\x08\"M\n\x13GatewayNotification\x12\x11\n\tpushUUIDs\x18\x01 \x03(\x0c\x12#\n\x05\x61larm\x18\x02 \x01(\x0b\x32\x14.CnAlarmNotification\"\x0b\n\tKeepAlive\" \n\x0c\x46\x61\x63toryReset\x12\x10\n\x08resetKey\x18\x01 \x02(\x0c\"D\n\x18SetDeviceSettingsRequest\x12\x12\n\nmacAddress\x18\x01 \x02(\x0c\x12\x14\n\x0cserialNumber\x18\x02 \x02(\t\"\x1a\n\x18SetDeviceSettingsConfirm\"!\n\x11SetAddressRequest\x12\x0c\n\x04uuid\x18\x01 \x02(\x0c\"\x13\n\x11SetAddressConfirm\"C\n\x12RegisterAppRequest\x12\x0c\n\x04uuid\x18\x01 \x02(\x0c\x12\x0b\n\x03pin\x18\x02 \x02(\r\x12\x12\n\ndevicename\x18\x03 \x02(\t\"\x14\n\x12RegisterAppConfirm\"\'\n\x13StartSessionRequest\x12\x10\n\x08takeover\x18\x01 \x01(\x08\":\n\x13StartSessionConfirm\x12\x12\n\ndevicename\x18\x01 \x01(\t\x12\x0f\n\x07resumed\x18\x02 \x01(\x08\"\x15\n\x13\x43loseSessionRequest\"\x15\n\x13\x43loseSessionConfirm\"\x1b\n\x19ListRegisteredAppsRequest\"r\n\x19ListRegisteredAppsConfirm\x12,\n\x04\x61pps\x18\x01 \x03(\x0b\x32\x1e.ListRegisteredAppsConfirm.App\x1a\'\n\x03\x41pp\x12\x0c\n\x04uuid\x18\x01 \x02(\x0c\x12\x12\n\ndevicename\x18\x02 \x02(\t\"$\n\x14\x44\x65registerAppRequest\x12\x0c\n\x04uuid\x18\x01 \x02(\x0c\"\x16\n\x14\x44\x65registerAppConfirm\"2\n\x10\x43hangePinRequest\x12\x0e\n\x06oldpin\x18\x01 \x02(\r\x12\x0e\n\x06newpin\x18\x02 \x02(\r\"\x12\n\x10\x43hangePinConfirm\"\x1a\n\x18GetRemoteAccessIdRequest\"(\n\x18GetRemoteAccessIdConfirm\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"(\n\x18SetRemoteAccessIdRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"\x1a\n\x18SetRemoteAccessIdConfirm\"\x15\n\x13GetSupportIdRequest\":\n\x13GetSupportIdConfirm\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x15\n\rremainingTime\x18\x02 \x01(\r\"6\n\x13SetSupportIdRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x11\n\tvalidTime\x18\x02 \x01(\r\"\x15\n\x13SetSupportIdConfirm\"\x11\n\x0fGetWebIdRequest\"\x1f\n\x0fGetWebIdConfirm\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"\x1f\n\x0fSetWebIdRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"\x11\n\x0fSetWebIdConfirm\" \n\x10SetPushIdRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"\x12\n\x10SetPushIdConfirm\"\xc0\x01\n\x0eUpgradeRequest\x12\x36\n\x07\x63ommand\x18\x01 \x01(\x0e\x32%.UpgradeRequest.UpgradeRequestCommand\x12\r\n\x05\x63hunk\x18\x02 \x01(\x0c\"g\n\x15UpgradeRequestCommand\x12\x11\n\rUPGRADE_START\x10\x00\x12\x14\n\x10UPGRADE_CONTINUE\x10\x01\x12\x12\n\x0eUPGRADE_FINISH\x10\x02\x12\x11\n\rUPGRADE_ABORT\x10\x03\"\x10\n\x0eUpgradeConfirm\"\xba\x03\n\x0c\x44\x65\x62ugRequest\x12\x32\n\x07\x63ommand\x18\x01 \x02(\x0e\x32!.DebugRequest.DebugRequestCommand\x12\x10\n\x08\x61rgument\x18\x02 \x01(\x05\"\xe3\x02\n\x13\x44\x65\x62ugRequestCommand\x12\x0c\n\x08\x44\x42G_ECHO\x10\x00\x12\r\n\tDBG_SLEEP\x10\x01\x12\x14\n\x10\x44\x42G_SESSION_ECHO\x10\x02\x12\x16\n\x12\x44\x42G_PRINT_SETTINGS\x10\x03\x12\r\n\tDBG_ALARM\x10\x04\x12\x0b\n\x07\x44\x42G_LED\x10\x05\x12\x0b\n\x07\x44\x42G_GPI\x10\x06\x12\x0b\n\x07\x44\x42G_GPO\x10\x07\x12\x13\n\x0f\x44\x42G_RS232_WRITE\x10\x08\x12\x12\n\x0e\x44\x42G_RS232_READ\x10\t\x12\x11\n\rDBG_CAN_WRITE\x10\n\x12\x10\n\x0c\x44\x42G_CAN_READ\x10\x0b\x12\x11\n\rDBG_KNX_WRITE\x10\x0c\x12\x10\n\x0c\x44\x42G_KNX_READ\x10\r\x12\x0e\n\nDBG_TOGGLE\x10\x0e\x12\x0e\n\nDBG_REBOOT\x10\x0f\x12\r\n\tDBG_CLOUD\x10\x10\x12\x13\n\x0f\x44\x42G_EEPROM_READ\x10\x11\x12\x14\n\x10\x44\x42G_EEPROM_WRITE\x10\x12\"\x1e\n\x0c\x44\x65\x62ugConfirm\x12\x0e\n\x06result\x18\x01 \x02(\x05\"\x10\n\x0eVersionRequest\"W\n\x0eVersionConfirm\x12\x16\n\x0egatewayVersion\x18\x01 \x02(\r\x12\x14\n\x0cserialNumber\x18\x02 \x02(\t\x12\x17\n\x0f\x63omfoNetVersion\x18\x03 \x02(\r\" \n\rCnTimeRequest\x12\x0f\n\x07setTime\x18\x01 \x01(\r\"$\n\rCnTimeConfirm\x12\x13\n\x0b\x63urrentTime\x18\x01 \x02(\r\"\x0f\n\rCnNodeRequest\"\xcf\x01\n\x12\x43nNodeNotification\x12\x0e\n\x06nodeId\x18\x01 \x02(\r\x12\x14\n\tproductId\x18\x02 \x01(\r:\x01\x30\x12\x0e\n\x06zoneId\x18\x03 \x01(\r\x12.\n\x04mode\x18\x04 \x01(\x0e\x32 .CnNodeNotification.NodeModeType\"S\n\x0cNodeModeType\x12\x0f\n\x0bNODE_LEGACY\x10\x00\x12\x10\n\x0cNODE_OFFLINE\x10\x01\x12\x0f\n\x0bNODE_NORMAL\x10\x02\x12\x0f\n\x0bNODE_UPDATE\x10\x03\"/\n\x0c\x43nRmiRequest\x12\x0e\n\x06nodeId\x18\x01 \x02(\r\x12\x0f\n\x07message\x18\x02 \x02(\x0c\"3\n\rCnRmiResponse\x12\x11\n\x06result\x18\x01 \x01(\r:\x01\x30\x12\x0f\n\x07message\x18\x02 \x01(\x0c\"4\n\x11\x43nRmiAsyncRequest\x12\x0e\n\x06nodeId\x18\x01 \x02(\r\x12\x0f\n\x07message\x18\x02 \x02(\x0c\"&\n\x11\x43nRmiAsyncConfirm\x12\x11\n\x06result\x18\x01 \x01(\r:\x01\x30\"8\n\x12\x43nRmiAsyncResponse\x12\x11\n\x06result\x18\x01 \x01(\r:\x01\x30\x12\x0f\n\x07message\x18\x02 \x01(\x0c\"[\n\rCnRpdoRequest\x12\x0c\n\x04pdid\x18\x01 \x02(\r\x12\x11\n\x04zone\x18\x02 \x01(\r:\x03\x32\x35\x35\x12\x0c\n\x04type\x18\x03 \x01(\r\x12\x1b\n\x07timeout\x18\x04 \x01(\r:\n4294967295\"\x0f\n\rCnRpdoConfirm\"0\n\x12\x43nRpdoNotification\x12\x0c\n\x04pdid\x18\x01 \x02(\r\x12\x0c\n\x04\x64\x61ta\x18\x02 \x02(\x0c\"\xaf\x01\n\x13\x43nAlarmNotification\x12\x0c\n\x04zone\x18\x01 \x01(\r\x12\x11\n\tproductId\x18\x02 \x01(\r\x12\x16\n\x0eproductVariant\x18\x03 \x01(\r\x12\x14\n\x0cserialNumber\x18\x04 \x01(\t\x12\x18\n\x10swProgramVersion\x18\x05 \x01(\r\x12\x0e\n\x06\x65rrors\x18\x06 \x01(\x0c\x12\x0f\n\x07\x65rrorId\x18\x07 \x01(\r\x12\x0e\n\x06nodeId\x18\x08 \x01(\r\"K\n\x18\x43nFupReadRegisterRequest\x12\x0c\n\x04node\x18\x01 \x02(\r\x12\x12\n\nregisterId\x18\x02 \x02(\r\x12\r\n\x05index\x18\x03 \x01(\r\")\n\x18\x43nFupReadRegisterConfirm\x12\r\n\x05value\x18\x01 \x02(\r\":\n\x18\x43nFupProgramBeginRequest\x12\x0c\n\x04node\x18\x01 \x03(\r\x12\x10\n\x05\x62lock\x18\x02 \x01(\r:\x01\x30\"\x1a\n\x18\x43nFupProgramBeginConfirm\"$\n\x13\x43nFupProgramRequest\x12\r\n\x05\x63hunk\x18\x01 \x02(\x0c\"\x15\n\x13\x43nFupProgramConfirm\"\x18\n\x16\x43nFupProgramEndRequest\"\x18\n\x16\x43nFupProgramEndConfirm\"2\n\x10\x43nFupReadRequest\x12\x0c\n\x04node\x18\x01 \x02(\r\x12\x10\n\x05\x62lock\x18\x02 \x01(\r:\x01\x30\"6\n\x10\x43nFupReadConfirm\x12\r\n\x05\x63hunk\x18\x01 \x01(\x0c\x12\x13\n\x04last\x18\x02 \x01(\x08:\x05\x66\x61lse\"!\n\x11\x43nFupResetRequest\x12\x0c\n\x04node\x18\x01 \x02(\r\"\x13\n\x11\x43nFupResetConfirm')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zehnder_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DISCOVERYOPERATION._serialized_start=18
  _DISCOVERYOPERATION._serialized_end=146
  _SEARCHGATEWAYREQUEST._serialized_start=148
  _SEARCHGATEWAYREQUEST._serialized_end=170
  _SEARCHGATEWAYRESPONSE._serialized_start=172
  _SEARCHGATEWAYRESPONSE._serialized_end=245
  _GATEWAYOPERATION._serialized_start=248
  _GATEWAYOPERATION._serialized_end=2390
  _GATEWAYOPERATION_OPERATIONTYPE._serialized_start=411
  _GATEWAYOPERATION_OPERATIONTYPE._serialized_end=2224
  _GATEWAYOPERATION_GATEWAYRESULT._serialized_start=2227
  _GATEWAYOPERATION_GATEWAYRESULT._serialized_end=2390
  _GATEWAYNOTIFICATION._serialized_start=2392
  _GATEWAYNOTIFICATION._serialized_end=2469
  _KEEPALIVE._serialized_start=2471
  _KEEPALIVE._serialized_end=2482
  _FACTORYRESET._serialized_start=2484
  _FACTORYRESET._serialized_end=2516
  _SETDEVICESETTINGSREQUEST._serialized_start=2518
  _SETDEVICESETTINGSREQUEST._serialized_end=2586
  _SETDEVICESETTINGSCONFIRM._serialized_start=2588
  _SETDEVICESETTINGSCONFIRM._serialized_end=2614
  _SETADDRESSREQUEST._serialized_start=2616
  _SETADDRESSREQUEST._serialized_end=2649
  _SETADDRESSCONFIRM._serialized_start=2651
  _SETADDRESSCONFIRM._serialized_end=2670
  _REGISTERAPPREQUEST._serialized_start=2672
  _REGISTERAPPREQUEST._serialized_end=2739
  _REGISTERAPPCONFIRM._serialized_start=2741
  _REGISTERAPPCONFIRM._serialized_end=2761
  _STARTSESSIONREQUEST._serialized_start=2763
  _STARTSESSIONREQUEST._serialized_end=2802
  _STARTSESSIONCONFIRM._serialized_start=2804
  _STARTSESSIONCONFIRM._serialized_end=2862
  _CLOSESESSIONREQUEST._serialized_start=2864
  _CLOSESESSIONREQUEST._serialized_end=2885
  _CLOSESESSIONCONFIRM._serialized_start=2887
  _CLOSESESSIONCONFIRM._serialized_end=2908
  _LISTREGISTEREDAPPSREQUEST._serialized_start=2910
  _LISTREGISTEREDAPPSREQUEST._serialized_end=2937
  _LISTREGISTEREDAPPSCONFIRM._serialized_start=2939
  _LISTREGISTEREDAPPSCONFIRM._serialized_end=3053
  _LISTREGISTEREDAPPSCONFIRM_APP._serialized_start=3014
  _LISTREGISTEREDAPPSCONFIRM_APP._serialized_end=3053
  _DEREGISTERAPPREQUEST._serialized_start=3055
  _DEREGISTERAPPREQUEST._serialized_end=3091
  _DEREGISTERAPPCONFIRM._serialized_start=3093
  _DEREGISTERAPPCONFIRM._serialized_end=3115
  _CHANGEPINREQUEST._serialized_start=3117
  _CHANGEPINREQUEST._serialized_end=3167
  _CHANGEPINCONFIRM._serialized_start=3169
  _CHANGEPINCONFIRM._serialized_end=3187
  _GETREMOTEACCESSIDREQUEST._serialized_start=3189
  _GETREMOTEACCESSIDREQUEST._serialized_end=3215
  _GETREMOTEACCESSIDCONFIRM._serialized_start=3217
  _GETREMOTEACCESSIDCONFIRM._serialized_end=3257
  _SETREMOTEACCESSIDREQUEST._serialized_start=3259
  _SETREMOTEACCESSIDREQUEST._serialized_end=3299
  _SETREMOTEACCESSIDCONFIRM._serialized_start=3301
  _SETREMOTEACCESSIDCONFIRM._serialized_end=3327
  _GETSUPPORTIDREQUEST._serialized_start=3329
  _GETSUPPORTIDREQUEST._serialized_end=3350
  _GETSUPPORTIDCONFIRM._serialized_start=3352
  _GETSUPPORTIDCONFIRM._serialized_end=3410
  _SETSUPPORTIDREQUEST._serialized_start=3412
  _SETSUPPORTIDREQUEST._serialized_end=3466
  _SETSUPPORTIDCONFIRM._serialized_start=3468
  _SETSUPPORTIDCONFIRM._serialized_end=3489
  _GETWEBIDREQUEST._serialized_start=3491
  _GETWEBIDREQUEST._serialized_end=3508
  _GETWEBIDCONFIRM._serialized_start=3510
  _GETWEBIDCONFIRM._serialized_end=3541
  _SETWEBIDREQUEST._serialized_start=3543
  _SETWEBIDREQUEST._serialized_end=3574
  _SETWEBIDCONFIRM._serialized_start=3576
  _SETWEBIDCONFIRM._serialized_end=3593
  _SETPUSHIDREQUEST._serialized_start=3595
  _SETPUSHIDREQUEST._serialized_end=3627
  _SETPUSHIDCONFIRM._serialized_start=3629
  _SETPUSHIDCONFIRM._serialized_end=3647
  _UPGRADEREQUEST._serialized_start=3650
  _UPGRADEREQUEST._serialized_end=3842
  _UPGRADEREQUEST_UPGRADEREQUESTCOMMAND._serialized_start=3739
  _UPGRADEREQUEST_UPGRADEREQUESTCOMMAND._serialized_end=3842
  _UPGRADECONFIRM._serialized_start=3844
  _UPGRADECONFIRM._serialized_end=3860
  _DEBUGREQUEST._serialized_start=3863
  _DEBUGREQUEST._serialized_end=4305
  _DEBUGREQUEST_DEBUGREQUESTCOMMAND._serialized_start=3950
  _DEBUGREQUEST_DEBUGREQUESTCOMMAND._serialized_end=4305
  _DEBUGCONFIRM._serialized_start=4307
  _DEBUGCONFIRM._serialized_end=4337
  _VERSIONREQUEST._serialized_start=4339
  _VERSIONREQUEST._serialized_end=4355
  _VERSIONCONFIRM._serialized_start=4357
  _VERSIONCONFIRM._serialized_end=4444
  _CNTIMEREQUEST._serialized_start=4446
  _CNTIMEREQUEST._serialized_end=4478
  _CNTIMECONFIRM._serialized_start=4480
  _CNTIMECONFIRM._serialized_end=4516
  _CNNODEREQUEST._serialized_start=4518
  _CNNODEREQUEST._serialized_end=4533
  _CNNODENOTIFICATION._serialized_start=4536
  _CNNODENOTIFICATION._serialized_end=4743
  _CNNODENOTIFICATION_NODEMODETYPE._serialized_start=4660
  _CNNODENOTIFICATION_NODEMODETYPE._serialized_end=4743
  _CNRMIREQUEST._serialized_start=4745
  _CNRMIREQUEST._serialized_end=4792
  _CNRMIRESPONSE._serialized_start=4794
  _CNRMIRESPONSE._serialized_end=4845
  _CNRMIASYNCREQUEST._serialized_start=4847
  _CNRMIASYNCREQUEST._serialized_end=4899
  _CNRMIASYNCCONFIRM._serialized_start=4901
  _CNRMIASYNCCONFIRM._serialized_end=4939
  _CNRMIASYNCRESPONSE._serialized_start=4941
  _CNRMIASYNCRESPONSE._serialized_end=4997
  _CNRPDOREQUEST._serialized_start=4999
  _CNRPDOREQUEST._serialized_end=5090
  _CNRPDOCONFIRM._serialized_start=5092
  _CNRPDOCONFIRM._serialized_end=5107
  _CNRPDONOTIFICATION._serialized_start=5109
  _CNRPDONOTIFICATION._serialized_end=5157
  _CNALARMNOTIFICATION._serialized_start=5160
  _CNALARMNOTIFICATION._serialized_end=5335
  _CNFUPREADREGISTERREQUEST._serialized_start=5337
  _CNFUPREADREGISTERREQUEST._serialized_end=5412
  _CNFUPREADREGISTERCONFIRM._serialized_start=5414
  _CNFUPREADREGISTERCONFIRM._serialized_end=5455
  _CNFUPPROGRAMBEGINREQUEST._serialized_start=5457
  _CNFUPPROGRAMBEGINREQUEST._serialized_end=5515
  _CNFUPPROGRAMBEGINCONFIRM._serialized_start=5517
  _CNFUPPROGRAMBEGINCONFIRM._serialized_end=5543
  _CNFUPPROGRAMREQUEST._serialized_start=5545
  _CNFUPPROGRAMREQUEST._serialized_end=5581
  _CNFUPPROGRAMCONFIRM._serialized_start=5583
  _CNFUPPROGRAMCONFIRM._serialized_end=5604
  _CNFUPPROGRAMENDREQUEST._serialized_start=5606
  _CNFUPPROGRAMENDREQUEST._serialized_end=5630
  _CNFUPPROGRAMENDCONFIRM._serialized_start=5632
  _CNFUPPROGRAMENDCONFIRM._serialized_end=5656
  _CNFUPREADREQUEST._serialized_start=5658
  _CNFUPREADREQUEST._serialized_end=5708
  _CNFUPREADCONFIRM._serialized_start=5710
  _CNFUPREADCONFIRM._serialized_end=5764
  _CNFUPRESETREQUEST._serialized_start=5766
  _CNFUPRESETREQUEST._serialized_end=5799
  _CNFUPRESETCONFIRM._serialized_start=5801
  _CNFUPRESETCONFIRM._serialized_end=5820
# @@protoc_insertion_point(module_scope)
